package com.jiuqi.office.excel;

import java.util.Iterator;
import java.util.Collection;
import org.apache.poi.hssf.util.HSSFColor;
import java.util.Hashtable;
import java.util.ArrayList;
import java.util.Map;
import java.util.List;

public class HSSFHelper
{
    private static final int[] WIN32_COLORS;
    private static List colorElements;
    private static Map colorFinder;
    
    static {
        WIN32_COLORS = new int[] { 13158600, 0, 13743257, 14405055, 15790320, 16777215, 6579300, 0, 0, 0, 11842740, 16578548, 11250603, 16750899, 16777215, 15790320, 10526880, 8421504, 0, 5525059, 16777215, 6908265, 14935011, 0, 14811135, 0, 13395456, 15389113, 15918295, 16750899, 15790320, 0 };
        HSSFHelper.colorElements = new ArrayList();
        HSSFHelper.colorFinder = new Hashtable();
    }
    
    private HSSFHelper() {
    }
    
    public static int pixelToWidth(int value) {
        if (value <= 0) {
            return 0;
        }
        if (value == 1) {
            return 19;
        }
        int width = 19;
        --value;
        width += value / 7 * 256;
        final int delta = value % 7;
        if (delta > 0) {
            width += delta / 2 * 73;
            if (delta % 2 == 1) {
                width += 37;
            }
        }
        return width;
    }
    
    public static int widthToPixel(final short value) {
        return (short)(value / 32);
    }
    
    public static short pixelToHeight(final int value) {
        if (value <= 0) {
            return 0;
        }
        final int height = 7 + (value - 1) * 15;
        return (short)height;
    }
    
    public static int heightToPixel(final short value) {
        return value / 15;
    }
    
    public static HSSFColor getApproximateColor(final int rgbColor) {
        final Integer key = new Integer(rgbColor);
        HSSFColor retColor = HSSFHelper.colorFinder.get(key);
        if (retColor == null) {
            retColor = findApproximateColor(rgbColor);
            if (retColor != null) {
                HSSFHelper.colorFinder.put(key, retColor);
            }
        }
        return retColor;
    }
    
    public static int getRGBColor(final HSSFColor hssfColor) {
        final short[] rgb = hssfColor.getTriplet();
        return rgb[0] << 16 | rgb[1] << 8 | rgb[2];
    }
    
    public static int getRGBColor(final short index) {
        final HSSFColor hssfColor = HSSFColor.getIndexHash().get(new Integer(index));
        if (hssfColor != null) {
            return getRGBColor(hssfColor);
        }
        return -1;
    }
    
    public static HSSFColor getApproximateTextColor(int rgbColor) {
        if (rgbColor == -16777211) {
            rgbColor = 0;
        }
        return getApproximateColor(rgbColor);
    }
    
    public static HSSFColor getApproximateEdgeColor(int rgbColor) {
        if (rgbColor == -16777201) {
            rgbColor = 8421504;
        }
        return getApproximateColor(rgbColor);
    }
    
    private static void buildColorList() {
        synchronized (HSSFHelper.colorElements) {
            if (HSSFHelper.colorElements.size() == 0) {
                Map colorMaps = null;
                colorMaps = HSSFColor.getIndexHash();
                final Collection colorList = colorMaps.values();
                final Iterator colorItr = colorList.iterator();
                while (colorItr.hasNext()) {
                    final ColorElement element = new ColorElement();
                    element.color = colorItr.next();
                    final short[] rgbVals = element.color.getTriplet();
                    element.rgb = rgbToColor(rgbVals[0], rgbVals[1], rgbVals[2]);
                    element.hsb = color2HSB(element.rgb);
                    HSSFHelper.colorElements.add(element);
                }
            }
            monitorexit(HSSFHelper.colorElements);
        }
    }
    
    private static HSSFColor findApproximateColor(final int rgbColor) {
        buildColorList();
        final HSB Hsb = color2HSB(rgbColor);
        double L = Math.sqrt(20000.0);
        ColorElement flagElement = null;
        for (int i = 0; i < HSSFHelper.colorElements.size(); ++i) {
            final ColorElement elem = HSSFHelper.colorElements.get(i);
            if (rgbColor == elem.rgb) {
                return elem.color;
            }
            double Q = Hsb.h - elem.hsb.h;
            if (Q > 180.0) {
                Q = 360.0 - Q;
            }
            final double E = Math.sqrt(Math.pow(elem.hsb.s - Hsb.s, 2.0) + Math.pow(elem.hsb.b - Hsb.b * Math.cos(Q * 3.141592653589793 / 180.0), 2.0) + Math.pow(Hsb.b * Math.sin(Q * 3.141592653589793 / 180.0), 2.0));
            if (L > E) {
                L = E;
                flagElement = elem;
            }
        }
        if (flagElement == null) {
            return null;
        }
        return flagElement.color;
    }
    
    private static int rgbToColor(int r, int g, final int b) {
        r <<= 16;
        g <<= 8;
        return r | g | b;
    }
    
    private static HSB color2HSB(final int value) {
        final HSB hsb = new HSB();
        final RGB rgb = colorToRGB(value);
        final double M0 = maxColorValue(rgb) / 2.55;
        hsb.setDefault();
        hsb.b = M0;
        if (hsb.b == 0.0) {
            hsb.s = 0.0;
        }
        else {
            final double M2 = minColorValue(rgb) / 2.55;
            hsb.s = (M0 - M2) * 100.0 / M0;
        }
        hsb.h = RGBToHue(rgb);
        return hsb;
    }
    
    private static RGB colorToRGB(int value) {
        if (value < 0) {
            value = HSSFHelper.WIN32_COLORS[value & 0xFF];
        }
        final int r = (value & 0xFF0000) >> 16;
        final int g = (value & 0xFF00) >> 8;
        final int b = value & 0xFF;
        return new RGB(r, g, b);
    }
    
    private static double RGBToHue(final RGB rgb) {
        final int M0 = minColorValue(rgb);
        final int M2 = maxColorValue(rgb);
        if (M0 == M2) {
            return 0.0;
        }
        final int M3 = centerColorValue(rgb);
        if (M2 != M3) {
            final double F = 60 * (M3 - M0) / (M2 - M0);
            double H;
            if (rgb.r == M2) {
                if (rgb.g < rgb.b) {
                    H = -F;
                }
                else {
                    H = F;
                }
            }
            else if (rgb.g == M2) {
                if (rgb.b < rgb.r) {
                    H = -F;
                }
                else {
                    H = F;
                }
                H += 120.0;
            }
            else {
                if (rgb.r < rgb.g) {
                    H = -F;
                }
                else {
                    H = F;
                }
                H += 240.0;
            }
            if (H < 0.0) {
                H += 360.0;
            }
            return H;
        }
        if (rgb.r == rgb.g) {
            return 60.0;
        }
        if (rgb.g == rgb.b) {
            return 180.0;
        }
        return 300.0;
    }
    
    private static int maxColorValue(final RGB rgb) {
        int max = rgb.r;
        if (rgb.g > max) {
            max = rgb.g;
        }
        if (rgb.b > max) {
            max = rgb.b;
        }
        return max;
    }
    
    private static int minColorValue(final RGB rgb) {
        int min = rgb.r;
        if (rgb.g < min) {
            min = rgb.g;
        }
        if (rgb.b < min) {
            min = rgb.b;
        }
        return min;
    }
    
    private static int centerColorValue(final RGB rgb) {
        if (rgb.r < rgb.g && rgb.r < rgb.b) {
            if (rgb.g < rgb.b) {
                return rgb.g;
            }
            return rgb.b;
        }
        else if (rgb.g < rgb.r && rgb.g < rgb.b) {
            if (rgb.r < rgb.b) {
                return rgb.r;
            }
            return rgb.b;
        }
        else {
            if (rgb.r < rgb.g) {
                return rgb.r;
            }
            return rgb.g;
        }
    }
    
    private static class RGB
    {
        public int r;
        public int g;
        public int b;
        
        public RGB() {
        }
        
        public RGB(final int r, final int g, final int b) {
            this.r = r;
            this.g = g;
            this.b = b;
        }
    }
    
    protected static class HSB
    {
        public double h;
        public double s;
        public double b;
        
        public void setDefault() {
            this.h = 65535.0;
            this.s = 0.0;
            this.b = 0.0;
        }
    }
    
    protected static class ColorElement
    {
        public int rgb;
        public HSB hsb;
        public HSSFColor color;
    }
}
